@@ -39,6 +39,7 @@ struct _SpCaptureReader
   int                  fd;
   gint                 endian;
   SpCaptureFileHeader  header;
+  gint64               end_time;
 };
 
 #ifndef SP_DISABLE_GOBJECT
@@ -297,6 +298,9 @@ sp_capture_reader_peek_frame (SpCaptureReader *self,
 
   sp_capture_reader_bswap_frame (self, frame);
 
+  if (frame->time > self->end_time)
+    self->end_time = frame->time;
+
   return TRUE;
 }
 
@@ -812,3 +816,68 @@ sp_capture_reader_get_start_time (SpCaptureReader *self)
 
   return self->header.time;
 }
+
+/**
+ * sp_capture_reader_get_end_time:
+ *
+ * This function will return the end time for the capture, which is the
+ * same as the last event added to the capture.
+ *
+ * If the end time has been stored in the capture header, that will be used.
+ * Otherwise, the time is discovered from the last capture frame and therefore
+ * the caller must have read all frames to ensure this value is accurate.
+ *
+ * Captures created by sysprof versions containing this API will have the
+ * end time set if the output capture file-descriptor supports seeking.
+ *
+ * Since: 3.22.1
+ */
+gint64
+sp_capture_reader_get_end_time (SpCaptureReader *self)
+{
+  g_return_val_if_fail (self != NULL, 0);
+
+  if (self->header.end_time != 0)
+    {
+      if (self->endian != G_BYTE_ORDER)
+        return GUINT64_SWAP_LE_BE (self->header.end_time);
+      return self->header.end_time;
+    }
+
+  return self->end_time;
+}
+
+/**
+ * sp_capture_reader_copy:
+ *
+ * This function makes a copy of the reader. Since readers use
+ * positioned reads with pread(), this allows you to have multiple
+ * readers with the shared file descriptor. This uses dup() to create
+ * another file descriptor.
+ *
+ * Returns: (transfer full): A copy of @self with a new file-descriptor.
+ */
+SpCaptureReader *
+sp_capture_reader_copy (SpCaptureReader *self)
+{
+  SpCaptureReader *copy;
+  int fd;
+
+  g_return_val_if_fail (self != NULL, NULL);
+
+  if (-1 == (fd = dup (self->fd)))
+    return NULL;
+
+  copy = g_new0 (SpCaptureReader, 1);
+
+  *copy = *self;
+
+  copy->ref_count = 1;
+  copy->filename = g_strdup (self->filename);
+  copy->fd = fd;
+
+  copy->buf = g_malloc (self->bufsz);
+  memcpy (copy->buf, self->buf, self->bufsz);
+
+  return copy;
+}
