@@ -26,12 +26,15 @@ G_BEGIN_DECLS
 #define SYSPROF_INSIDE
 # include "sp-address.h"
 # include "sp-callgraph-profile.h"
+# include "sp-capture-condition.h"
+# include "sp-capture-cursor.h"
 # include "sp-capture-reader.h"
 # include "sp-capture-writer.h"
 # include "sp-clock.h"
 # include "sp-elf-symbol-resolver.h"
 # include "sp-error.h"
 # include "sp-gjs-source.h"
+# include "sp-hostinfo-source.h"
 # include "sp-jitmap-symbol-resolver.h"
 # include "sp-kernel-symbol-resolver.h"
 # include "sp-kernel-symbol.h"
@@ -41,6 +44,7 @@ G_BEGIN_DECLS
 # include "sp-proc-source.h"
 # include "sp-profile.h"
 # include "sp-profiler.h"
+# include "sp-selection.h"
 # include "sp-source.h"
 # include "sp-symbol-resolver.h"
 # include "sysprof-version.h"
