@@ -18,10 +18,13 @@
 
 #include <errno.h>
 #include <glib/gi18n.h>
+#include <math.h>
 #include <sysprof.h>
 #include <sysprof-ui.h>
 
 #include "sp-application.h"
+#include "sp-multi-paned.h"
+#include "sp-visualizer-view.h"
 #include "sp-window.h"
 #include "sp-window-settings.h"
 
@@ -34,6 +37,8 @@ struct _SpWindow
   SpProfiler           *profiler;
   SpCaptureReader      *reader;
 
+  GCancellable         *refilter_cancellable;
+
   /* Gtk widget template children */
   SpCallgraphView      *callgraph_view;
   SpEmptyStateView     *empty_view;
@@ -41,6 +46,7 @@ struct _SpWindow
   GtkInfoBar           *info_bar;
   GtkLabel             *info_bar_label;
   GtkRevealer          *info_bar_revealer;
+  GtkPaned             *paned;
   SpProfilerMenuButton *profiler_menu_button;
   SpRecordingStateView *recording_view;
   GtkButton            *record_button;
@@ -48,6 +54,9 @@ struct _SpWindow
   GtkLabel             *stat_label;
   GtkLabel             *title;
   GtkStack             *view_stack;
+  SpVisualizerView     *visualizers;
+  SpZoomManager        *zoom_manager;
+  GtkLabel             *zoom_one_label;
 
   guint                 stats_handler;
 
@@ -205,32 +214,51 @@ sp_window_build_profile_cb (GObject      *object,
 
   if (!sp_profile_generate_finish (profile, result, &error))
     {
+      /* If we were cancelled while updating the selection, ignore the failure */
+      if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) &&
+          (self->state == SP_WINDOW_STATE_BROWSING))
+        return;
       sp_window_notify_user (self, GTK_MESSAGE_ERROR, "%s", error->message);
       sp_window_set_state (self, SP_WINDOW_STATE_EMPTY);
       return;
     }
 
   sp_callgraph_view_set_profile (self->callgraph_view, SP_CALLGRAPH_PROFILE (profile));
-  sp_window_set_state (self, SP_WINDOW_STATE_BROWSING);
-
   if (sp_callgraph_view_get_n_functions (self->callgraph_view) == 0)
     sp_window_notify_user (self,
                            GTK_MESSAGE_WARNING,
                            _("Not enough samples were collected to generate a callgraph"));
+
+  sp_visualizer_view_set_reader (self->visualizers, self->reader);
+
+  sp_window_set_state (self, SP_WINDOW_STATE_BROWSING);
 }
 
 static void
 sp_window_build_profile (SpWindow *self)
 {
   g_autoptr(SpProfile) profile = NULL;
+  SpSelection *selection;
 
   g_assert (SP_IS_WINDOW (self));
   g_assert (self->reader != NULL);
 
-  profile = sp_callgraph_profile_new ();
+  if (self->refilter_cancellable != NULL)
+    {
+      if (!g_cancellable_is_cancelled (self->refilter_cancellable))
+        g_cancellable_cancel (self->refilter_cancellable);
+      g_clear_object (&self->refilter_cancellable);
+    }
+
+  selection = sp_visualizer_view_get_selection (self->visualizers);
+
+  profile = sp_callgraph_profile_new_with_selection (selection);
   sp_profile_set_reader (profile, self->reader);
+
+  self->refilter_cancellable = g_cancellable_new ();
+
   sp_profile_generate (profile,
-                       NULL,
+                       self->refilter_cancellable,
                        sp_window_build_profile_cb,
                        g_object_ref (self));
 }
@@ -362,6 +390,7 @@ static void
 sp_window_add_sources (SpWindow   *window,
                        SpProfiler *profiler)
 {
+  g_autoptr(SpSource) host_source = NULL;
   g_autoptr(SpSource) proc_source = NULL;
   g_autoptr(SpSource) perf_source = NULL;
 
@@ -373,6 +402,9 @@ sp_window_add_sources (SpWindow   *window,
 
   perf_source = sp_perf_source_new ();
   sp_profiler_add_source (profiler, perf_source);
+
+  host_source = sp_hostinfo_source_new ();
+  sp_profiler_add_source (profiler, host_source);
 }
 
 static void
@@ -540,31 +572,32 @@ sp_window_save_capture (GSimpleAction *action,
 {
   g_autoptr(SpCaptureReader) reader = NULL;
   SpWindow *self = user_data;
-  GtkWidget *dialog;
-  GtkResponseType response;
+  GtkFileChooserNative *dialog;
+  gint response;
 
   g_assert (G_IS_SIMPLE_ACTION (action));
   g_assert (variant == NULL);
   g_assert (SP_IS_WINDOW (self));
 
   if (self->reader == NULL)
-    return;
+    {
+      g_warning ("Save called without a capture open, ignoring");
+      return;
+    }
 
   reader = sp_capture_reader_ref (self->reader);
 
-  dialog = gtk_file_chooser_dialog_new (_("Save Capture As"),
+  dialog = gtk_file_chooser_native_new (_("Save Capture As"),
                                         GTK_WINDOW (self),
                                         GTK_FILE_CHOOSER_ACTION_SAVE,
-                                        _("Cancel"), GTK_RESPONSE_CANCEL,
-                                        _("Save"), GTK_RESPONSE_OK,
-                                        NULL);
+                                        _("Save"),
+                                        _("Cancel"));
 
   gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (dialog), TRUE);
-  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
-  response = gtk_dialog_run (GTK_DIALOG (dialog));
+  response = gtk_native_dialog_run (GTK_NATIVE_DIALOG (dialog));
 
-  if (response == GTK_RESPONSE_OK)
+  if (response == GTK_RESPONSE_ACCEPT)
     {
       g_autofree gchar *filename = NULL;
       g_autoptr(GError) error = NULL;
@@ -595,7 +628,7 @@ sp_window_save_capture (GSimpleAction *action,
     }
 
 failure:
-  gtk_widget_destroy (dialog);
+  gtk_native_dialog_destroy (GTK_NATIVE_DIALOG (dialog));
 }
 
 static void
@@ -691,11 +724,39 @@ sp_window_delete_event (GtkWidget   *widget,
   return GDK_EVENT_PROPAGATE;
 }
 
+static gboolean
+zoom_level_to_string (GBinding     *binding,
+                      const GValue *from_value,
+                      GValue       *to_value,
+                      gpointer      user_data)
+{
+  gdouble percent = 100.0 * g_value_get_double (from_value);
+  g_value_take_string (to_value, g_strdup_printf ("%u%%", (guint)floor (percent)));
+  return TRUE;
+}
+
+static void
+sp_window_visualizers_selection_changed (SpWindow              *self,
+                                         SpSelection *selection)
+{
+  g_assert (SP_IS_WINDOW (self));
+  g_assert (SP_IS_SELECTION (selection));
+
+  sp_window_build_profile (self);
+}
+
 static void
 sp_window_destroy (GtkWidget *widget)
 {
   SpWindow *self = (SpWindow *)widget;
 
+  if (self->refilter_cancellable != NULL)
+    {
+      if (!g_cancellable_is_cancelled (self->refilter_cancellable))
+        g_cancellable_cancel (self->refilter_cancellable);
+      g_clear_object (&self->refilter_cancellable);
+    }
+
   g_clear_object (&self->profiler);
   g_clear_pointer (&self->reader, sp_capture_reader_unref);
   sp_window_disable_stats (self);
@@ -753,6 +814,7 @@ sp_window_class_init (SpWindowClass *klass)
   gtk_widget_class_bind_template_child (widget_class, SpWindow, info_bar);
   gtk_widget_class_bind_template_child (widget_class, SpWindow, info_bar_label);
   gtk_widget_class_bind_template_child (widget_class, SpWindow, info_bar_revealer);
+  gtk_widget_class_bind_template_child (widget_class, SpWindow, paned);
   gtk_widget_class_bind_template_child (widget_class, SpWindow, profiler_menu_button);
   gtk_widget_class_bind_template_child (widget_class, SpWindow, record_button);
   gtk_widget_class_bind_template_child (widget_class, SpWindow, recording_view);
@@ -760,6 +822,9 @@ sp_window_class_init (SpWindowClass *klass)
   gtk_widget_class_bind_template_child (widget_class, SpWindow, subtitle);
   gtk_widget_class_bind_template_child (widget_class, SpWindow, title);
   gtk_widget_class_bind_template_child (widget_class, SpWindow, view_stack);
+  gtk_widget_class_bind_template_child (widget_class, SpWindow, visualizers);
+  gtk_widget_class_bind_template_child (widget_class, SpWindow, zoom_manager);
+  gtk_widget_class_bind_template_child (widget_class, SpWindow, zoom_one_label);
 }
 
 static void
@@ -771,8 +836,7 @@ sp_window_init (SpWindow *self)
     { "save-capture",  sp_window_save_capture },
     { "screenshot",  sp_window_screenshot },
   };
-  GtkApplication *app;
-  GMenu *menu;
+  SpSelection *selection;
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
@@ -798,27 +862,34 @@ sp_window_init (SpWindow *self)
                            self,
                            G_CONNECT_SWAPPED);
 
+  g_object_bind_property_full (self->zoom_manager, "zoom", self->zoom_one_label, "label",
+                               G_BINDING_SYNC_CREATE,
+                               zoom_level_to_string, NULL, NULL, NULL);
+
+  /*
+   * Wire up selections for visualizers to update callgraph.
+   */
+
+  selection = sp_visualizer_view_get_selection (self->visualizers);
+
+  g_signal_connect_object (selection,
+                           "changed",
+                           G_CALLBACK (sp_window_visualizers_selection_changed),
+                           self,
+                           G_CONNECT_SWAPPED);
+
   /*
    * Setup actions for the window.
    */
-
   g_action_map_add_action_entries (G_ACTION_MAP (self),
                                    action_entries,
                                    G_N_ELEMENTS (action_entries),
                                    self);
-
-  /*
-   * Setup our gear (hamburger) menu.
-   */
-
-  app = GTK_APPLICATION (g_application_get_default ());
-  menu = gtk_application_get_menu_by_id (app, "gear-menu");
-  gtk_menu_button_set_menu_model (self->gear_menu_button, G_MENU_MODEL (menu));
+  gtk_widget_insert_action_group (GTK_WIDGET (self), "zoom", G_ACTION_GROUP (self->zoom_manager));
 
   /*
    * Restore previous window settings.
    */
-
   sp_window_settings_register (GTK_WINDOW (self));
 
   /*
@@ -916,23 +987,17 @@ sp_window_get_state (SpWindow *self)
 void
 sp_window_open_from_dialog (SpWindow *self)
 {
+  GtkFileChooserNative *dialog;
   GtkFileFilter *filter;
-  GtkDialog *dialog;
+  gint response;
 
   g_assert (SP_IS_WINDOW (self));
 
-  dialog = g_object_new (GTK_TYPE_FILE_CHOOSER_DIALOG,
-                         "action", GTK_FILE_CHOOSER_ACTION_OPEN,
-                         "title", _("Open Capture"),
-                         "transient-for", self,
-                         NULL);
-
-  gtk_dialog_add_buttons (dialog,
-                          _("Cancel"), GTK_RESPONSE_CANCEL,
-                          _("Open"), GTK_RESPONSE_OK,
-                          NULL);
-
-  gtk_dialog_set_default_response (dialog, GTK_RESPONSE_OK);
+  dialog = gtk_file_chooser_native_new (_("Open Capture"),
+                                        GTK_WINDOW (self),
+                                        GTK_FILE_CHOOSER_ACTION_OPEN,
+                                        _("Open"),
+                                        _("Cancel"));
 
   filter = gtk_file_filter_new ();
   gtk_file_filter_set_name (filter, _("Sysprof Captures"));
@@ -944,7 +1009,9 @@ sp_window_open_from_dialog (SpWindow *self)
   gtk_file_filter_add_pattern (filter, "*");
   gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
 
-  if (gtk_dialog_run (dialog) == GTK_RESPONSE_OK)
+  response = gtk_native_dialog_run (GTK_NATIVE_DIALOG (dialog));
+
+  if (response == GTK_RESPONSE_ACCEPT)
     {
       g_autoptr(GFile) file = NULL;
 
@@ -952,5 +1019,5 @@ sp_window_open_from_dialog (SpWindow *self)
       sp_window_open (self, file);
     }
 
-  gtk_widget_destroy (GTK_WIDGET (dialog));
+  gtk_native_dialog_destroy (GTK_NATIVE_DIALOG (dialog));
 }
