@@ -34,6 +34,8 @@ test_reader_basic (void)
   writer = sp_capture_writer_new ("capture-file", 0);
   g_assert (writer != NULL);
 
+  sp_capture_writer_flush (writer);
+
   reader = sp_capture_reader_new ("capture-file", &error);
   g_assert_no_error (error);
   g_assert (reader != NULL);
@@ -75,6 +77,11 @@ test_reader_basic (void)
       g_assert_cmpstr (map->filename, ==, str);
     }
 
+  /* Now that we have read a frame, we should start having updated
+   * end times with each incoming frame.
+   */
+  g_assert_cmpint (0, !=, sp_capture_reader_get_end_time (reader));
+
   for (i = 0; i < 100; i++)
     {
       r = sp_capture_writer_add_timestamp (writer, t, i, -1);
@@ -530,6 +537,7 @@ int
 main (int argc,
       char *argv[])
 {
+  sp_clock_init ();
   g_test_init (&argc, &argv, NULL);
   g_test_add_func ("/SpCapture/ReaderWriter", test_reader_basic);
   g_test_add_func ("/SpCapture/Writer/splice", test_writer_splice);
